import React, { useState } from "react";
import { fetchWithAuth } from "../utils/auth-client";
import "../styles/global.css";

interface FormData {
  name: string;
  attending: boolean | null;
  dietaryRestrictions: string;
  notes: string;
}

interface ApiResponse {
  success: boolean;
  error?: string;
}

const RSVPForm = () => {
  const [formData, setFormData] = useState<FormData>({
    name: "",
    attending: null,
    dietaryRestrictions: "",
    notes: "",
  });
  const [isSubmitting, setIsSubmitting] = useState(false);
  const [error, setError] = useState<string | null>(null);
  const [success, setSuccess] = useState(false);

  const isFormValid = formData.name.trim().length > 0 && formData.attending !== null;

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    if (!isFormValid) return;

    setIsSubmitting(true);
    setError(null);
    setSuccess(false);

    try {
      const controller = new AbortController();
      const timeoutId = setTimeout(() => controller.abort(), 10000);

      const response = await fetchWithAuth("/api/rsvp", {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify({
          ...formData,
          attending: formData.attending === true, // Ensure it's a boolean
          timestamp: new Date().toISOString(),
        }),
        signal: controller.signal,
      });

      clearTimeout(timeoutId);

      if (!response.ok) {
        const errorText = await response.text();
        console.error("Server response:", errorText);
        throw new Error(`HTTP error! status: ${response.status}`);
      }

      const data: ApiResponse = await response.json();

      if (!data.success) {
        throw new Error(data.error || "Failed to submit RSVP");
      }

      setFormData({
        name: "",
        attending: null,
        dietaryRestrictions: "",
        notes: "",
      });
      setSuccess(true);
    } catch (err: any) {
      if (err?.name === 'AbortError') {
        setError('Request timed out. Please try again.');
      } else {
        const errorMessage = (err instanceof Error && err.message) ? err.message : "Failed to submit RSVP. Please try again.";
        setError(errorMessage);
      }
      console.error("Error submitting RSVP:", err);
    } finally {
      setIsSubmitting(false);
    }
  };

  return (
    <div className="max-w-2xl mx-auto p-6">
      <div className="card bg-base-100 shadow-xl">
        <div className="card-body flex flex-col gap-6">
          <h2 className="card-title text-2xl font-bold text-center justify-center">
            Wedding RSVP
          </h2>

          {error && (
            <div className="alert alert-error">
              <svg
                xmlns="http://www.w3.org/2000/svg"
                className="stroke-current shrink-0 h-6 w-6"
                fill="none"
                viewBox="0 0 24 24"
              >
                <path
                  strokeLinecap="round"
                  strokeLinejoin="round"
                  strokeWidth="2"
                  d="M10 14l2-2m0 0l2-2m-2 2l-2-2m2 2l2 2m7-2a9 9 0 11-18 0 9 9 0 0118 0z"
                />
              </svg>
              <span>{error}</span>
            </div>
          )}

          {success && (
            <div className="alert alert-success">
              <svg
                xmlns="http://www.w3.org/2000/svg"
                className="stroke-current shrink-0 h-6 w-6"
                fill="none"
                viewBox="0 0 24 24"
              >
                <path
                  strokeLinecap="round"
                  strokeLinejoin="round"
                  strokeWidth="2"
                  d="M9 12l2 2 4-4m6 2a9 9 0 11-18 0 9 9 0 0118 0z"
                />
              </svg>
              <span>Thank you for your RSVP!</span>
            </div>
          )}

          <form onSubmit={handleSubmit} className="flex flex-col gap-6">
            {/* Name Input */}
            <div className="flex flex-col gap-2">
              <label htmlFor="name" className="label">
                <span className="label-text">Full Name *</span>
              </label>
              <input
                type="text"
                id="name"
                value={formData.name}
                onChange={(e) =>
                  setFormData((prev) => ({ ...prev, name: e.target.value }))
                }
                placeholder="Enter your full name"
                className="input input-bordered w-full"
                required
                disabled={isSubmitting}
              />
            </div>

            {/* Attendance Selection */}
            <div className="flex flex-col gap-2">
              <label className="label">
                <span className="label-text">Will you be attending? *</span>
              </label>
              <div className="flex gap-4">
                <label className="flex items-center gap-2 cursor-pointer">
                  <input
                    type="radio"
                    className="radio radio-primary"
                    checked={formData.attending === true}
                    onChange={() => setFormData(prev => ({ ...prev, attending: true }))}
                    disabled={isSubmitting}
                  />
                  <span>Yes, I'll be there!</span>
                </label>
                <label className="flex items-center gap-2 cursor-pointer">
                  <input
                    type="radio"
                    className="radio radio-primary"
                    checked={formData.attending === false}
                    onChange={() => setFormData(prev => ({ ...prev, attending: false }))}
                    disabled={isSubmitting}
                  />
                  <span>No, I can't make it</span>
                </label>
              </div>
            </div>

            {/* Only show these fields if attending */}
            {formData.attending && (
              <>
                {/* Dietary Restrictions Input */}
                <div className="flex flex-col gap-2">
                  <label htmlFor="dietary" className="label">
                    <span className="label-text">Dietary Restrictions</span>
                  </label>
                  <textarea
                    id="dietary"
                    value={formData.dietaryRestrictions}
                    onChange={(e) =>
                      setFormData((prev) => ({
                        ...prev,
                        dietaryRestrictions: e.target.value,
                      }))
                    }
                    placeholder="Please list any dietary restrictions or allergies"
                    className="textarea textarea-bordered h-24"
                    disabled={isSubmitting}
                  />
                </div>
              </>
            )}

            {/* Notes Input - shown for both attending and not attending */}
            <div className="flex flex-col gap-2">
              <label htmlFor="notes" className="label">
                <span className="label-text">
                  {formData.attending
                    ? "Anything else you'd like Ix and Tasha to know?"
                    : "Would you like to leave a message for Ix and Tasha?"}
                </span>
              </label>
              <textarea
                id="notes"
                value={formData.notes}
                onChange={(e) =>
                  setFormData((prev) => ({
                    ...prev,
                    notes: e.target.value,
                  }))
                }
                placeholder={formData.attending 
                  ? "e.g. attending just the ceremony or reception, etc."
                  : "Optional message"}
                className="textarea textarea-bordered h-24"
                disabled={isSubmitting}
              />
            </div>

            {/* Submit Button */}
            <button
              type="submit"
              className={`btn w-full ${formData.attending ? 'btn-primary' : 'btn-secondary'}`}
              disabled={!isFormValid || isSubmitting}
            >
              {isSubmitting ? (
                <>
                  <span className="loading loading-spinner"></span>
                  Submitting...
                </>
              ) : formData.attending ? (
                "Yes, I'll Be There!"
              ) : formData.attending === false ? (
                "Submit RSVP"
              ) : (
                "Submit RSVP"
              )}
            </button>
          </form>
        </div>
      </div>
    </div>
  );
};

export default RSVPForm;
