import { useState, useEffect } from "react";
import type { RSVPItem } from "../lib/types";
import { fetchWithAuth, getAuthToken } from "../utils/auth-client";

const RSVPManager = () => {
  const [rsvpList, setRSVPList] = useState<RSVPItem[]>([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);

  const fetchRSVPList = async () => {
    // Don't try to fetch if we don't have a token yet
    if (!getAuthToken()) {
      setError("No authentication token found");
      setLoading(false);
      return;
    }

    setLoading(true);
    setError(null);
    try {
      const response = await fetchWithAuth("/api/rsvp");
      if (!response.ok) {
        throw new Error(`HTTP error! status: ${response.status}`);
      }
      const data = await response.json();
      setRSVPList(data);
      setError(null);
    } catch (e: any) {
      setError(e.message);
      console.error("Failed to fetch RSVP list:", e);
    } finally {
      setLoading(false);
    }
  };

  const handleDeleteRSVP = async (name: string, timestamp: string) => {
    if (!confirm(`Are you sure you want to delete the RSVP from ${name}?`)) {
      return;
    }

    try {
      const response = await fetchWithAuth(`/api/rsvp`, {
        method: "DELETE",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify({ name, timestamp }),
      });

      if (!response.ok) {
        throw new Error(`Failed to delete RSVP: ${response.statusText}`);
      }

      await fetchRSVPList();
    } catch (error: any) {
      setError(error.message);
      console.error("Error deleting RSVP:", error);
    }
  };

  useEffect(() => {
    fetchRSVPList();

    // Add listener for auth success to retry fetching
    const handleAuthSuccess = () => {
      fetchRSVPList();
    };
    document.addEventListener("auth-success", handleAuthSuccess);
    return () => document.removeEventListener("auth-success", handleAuthSuccess);
  }, []);

  if (loading) {
    return <div className="text-center">Loading RSVP list...</div>;
  }

  if (error) {
    if (error === "No authentication token found") {
      return <div className="text-center">Initializing...</div>;
    }
    return <div className="text-red-500 text-center">Error: {error}</div>;
  }

  const attending = rsvpList.filter(rsvp => rsvp.attending === true);
  const notAttending = rsvpList.filter(rsvp => rsvp.attending === false);

  return (
    <div className="flex flex-col gap-4 mx-auto p-4">
      <div className="stats shadow mb-8 mx-auto">
        <div className="stat">
          <div className="stat-title">Total Responses</div>
          <div className="stat-value">{rsvpList.length}</div>
        </div>
        <div className="stat">
          <div className="stat-title">Attending</div>
          <div className="stat-value text-success">{attending.length}</div>
          <div className="stat-desc">Will be there</div>
        </div>
        <div className="stat">
          <div className="stat-title">Not Attending</div>
          <div className="stat-value text-error">{notAttending.length}</div>
          <div className="stat-desc">Can't make it</div>
        </div>
      </div>

      <div className="overflow-x-auto mx-auto">
        <table className="table w-full">
          <thead>
            <tr>
              <th>Name</th>
              <th>Status</th>
              <th>Dietary Restrictions</th>
              <th>Notes</th>
              <th>Response Date</th>
              <th>Actions</th>
            </tr>
          </thead>
          <tbody>
            {rsvpList.map((rsvp, index) => (
              <tr key={index}>
                <td>{rsvp.name}</td>
                <td>
                  <span className={`badge badge-lg ${
                    rsvp.attending ? 'badge-success' : 'badge-error'
                  } whitespace-nowrap`}>
                    {rsvp.attending ? 'Attending' : 'Not Attending'}
                  </span>
                </td>
                <td>{rsvp.dietaryRestrictions && rsvp.dietaryRestrictions !== "undefined" ? rsvp.dietaryRestrictions : "-"}</td>
                <td>{rsvp.notes && rsvp.notes !== "undefined" ? rsvp.notes : "-"}</td>
                <td className="whitespace-nowrap">{new Date(rsvp.timestamp).toLocaleDateString('en-US', {
                  year: 'numeric',
                  month: 'short',
                  day: 'numeric',
                  hour: '2-digit',
                  minute: '2-digit'
                })}</td>
                <td>
                  <button
                    onClick={() => handleDeleteRSVP(rsvp.name, rsvp.timestamp)}
                    className="btn btn-error btn-sm"
                  >
                    Delete
                  </button>
                </td>
              </tr>
            ))}
          </tbody>
        </table>
      </div>
    </div>
  );
};

export default RSVPManager; 