import React, { useState } from "react";
import { setAuthData } from "../utils/auth-client";

interface SignInProps {
  onSuccess?: () => void;
  requiredRole?: "guest" | "admin";
}

interface ApiResponse {
  success: boolean;
  error?: string;
  token?: string;
}

const SignIn = ({ onSuccess, requiredRole = "guest" }: SignInProps) => {
  const [code, setCode] = useState("");
  const [error, setError] = useState<string | null>(null);
  const [isSubmitting, setIsSubmitting] = useState(false);

  // Initialize disabled state to true since code is empty initially
  const isDisabled = !code || isSubmitting;

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    if (isDisabled) return;

    setIsSubmitting(true);
    setError(null);

    try {
      const response = await fetch("/api/auth", {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify({ code, requiredRole }),
      });

      const data: ApiResponse = await response.json();

      if (!response.ok || !data.success || !data.token) {
        throw new Error(data.error || "Invalid code");
      }

      setAuthData(data.token);
      onSuccess?.();
    } catch (err: any) {
      setError(err instanceof Error ? err.message : "Authentication failed");
    } finally {
      setIsSubmitting(false);
    }
  };

  return (
    <div className="max-w-2xl mx-auto p-6">
      <div className="card bg-base-100 shadow-xl">
        <div className="card-body flex flex-col gap-6">
          <h2 className="card-title text-2xl font-bold text-center justify-center">
            Enter Secret Code
          </h2>

          {error && (
            <div className="alert alert-error">
              <span>{error}</span>
            </div>
          )}

          <form onSubmit={handleSubmit} className="flex flex-col gap-6">
            <div className="flex flex-col gap-2">
              <input
                type="password"
                value={code}
                onChange={(e) => setCode(e.target.value)}
                placeholder="Enter secret code"
                className="input input-bordered w-full"
                required
                disabled={isSubmitting}
              />
            </div>

            <button
              type="submit"
              className="btn btn-primary w-full"
              disabled={isDisabled}
            >
              {isSubmitting ? "Verifying..." : "Continue"}
            </button>
          </form>
        </div>
      </div>
    </div>
  );
};

export default SignIn;
