import type { APIRoute } from "astro";
import { getS3Data, putS3Data } from "../../lib/s3";
import type { RSVPItem } from "../../lib/types";
import { createProtectedAPIRoute } from "../../utils/auth-middleware";

const objectsToCSV = (data: RSVPItem[]): string => {
  const headers = ["name", "attending", "dietaryRestrictions", "notes", "timestamp"];
  const csvRows = [headers.join(",")];

  data.forEach((entry) => {
    const row = headers.map((header) => {
      let field = entry[header as keyof RSVPItem];
      // Convert boolean to string for CSV
      if (typeof field === 'boolean') {
        field = field.toString();
      }
      // Handle null/undefined
      if (field === null || field === undefined) {
        field = '';
      }
      const escaped = String(field).replace(/"/g, '""');
      return `"${escaped}"`;
    });
    csvRows.push(row.join(","));
  });

  return csvRows.join("\n");
};

const csvToObjects = (csv: string): RSVPItem[] => {
  const lines = csv.split("\n");
  const headers = lines[0].split(",").map((h) => h.trim());
  const hasAttendingColumn = headers.includes("attending");

  return lines
    .slice(1)
    .filter((line) => line.trim())
    .map((line) => {
      const values = line.match(/(".*?"|[^",\s]+)(?=\s*,|\s*$)/g) || [];
      const entry: Partial<RSVPItem> = {};

      headers.forEach((header, index) => {
        let value = values[index] || "";
        value = value.replace(/^"(.*)"$/, "$1").replace(/""/g, '"');
        entry[header as keyof RSVPItem] = value;
      });

      // If the attending column doesn't exist in the CSV, assume they're attending if they provided dietary restrictions
      if (!hasAttendingColumn) {
        entry.attending = Boolean(entry.dietaryRestrictions);
      } else if (typeof entry.attending === 'string') {
        entry.attending = entry.attending.toLowerCase() === 'true';
      }

      return entry as RSVPItem;
    });
};

// GET: Retrieve all RSVPs (requires admin role)
const handleGet: APIRoute = async ({ request }) => {
  const headers = {
    "Content-Type": "application/json",
  };

  try {
    const FILE_KEY = "rsvp.csv";
    const fileContent = await getS3Data<string>(FILE_KEY);
    
    if (!fileContent) {
      return new Response(JSON.stringify([]), {
        status: 200,
        headers,
      });
    }

    const rsvpList = csvToObjects(fileContent);
    return new Response(JSON.stringify(rsvpList), {
      status: 200,
      headers,
    });
  } catch (error) {
    console.error("Error retrieving RSVPs:", error);
    return new Response(
      JSON.stringify({
        success: false,
        error: "Failed to retrieve RSVPs",
      }),
      {
        status: 500,
        headers,
      }
    );
  }
};

// POST: Submit a new RSVP (requires guest role)
const handlePost: APIRoute = async ({ request }) => {
  const headers = {
    "Content-Type": "application/json",
  };

  try {
    const FILE_KEY = "rsvp.csv";
    const newRsvp = await request.json();

    let existingRsvps: RSVPItem[] = [];
    const fileContent = await getS3Data<string>(FILE_KEY);
    if (fileContent) {
      existingRsvps = csvToObjects(fileContent);
    }

    existingRsvps.push({
      ...newRsvp,
      attending: Boolean(newRsvp.attending),
      notes: newRsvp.notes || "",
      dietaryRestrictions: newRsvp.dietaryRestrictions || "",
      timestamp: newRsvp.timestamp || new Date().toISOString(),
    });

    const csvContent = objectsToCSV(existingRsvps);
    await putS3Data(FILE_KEY, csvContent, "text/csv");

    return new Response(JSON.stringify({ success: true }), {
      status: 200,
      headers,
    });
  } catch (error) {
    console.error("Error handling RSVP:", error);
    return new Response(
      JSON.stringify({
        success: false,
        error: error instanceof Error ? error.message : "Unknown error",
      }),
      {
        status: 500,
        headers,
      }
    );
  }
};

// DELETE: Delete an RSVP (requires admin role)
const handleDelete: APIRoute = async ({ request }) => {
  const headers = {
    "Content-Type": "application/json",
  };

  try {
    const FILE_KEY = "rsvp.csv";
    const { name, timestamp } = await request.json();

    if (!name || !timestamp) {
      return new Response(
        JSON.stringify({
          success: false,
          error: "Name and timestamp are required",
        }),
        {
          status: 400,
          headers,
        }
      );
    }

    const fileContent = await getS3Data<string>(FILE_KEY);
    if (!fileContent) {
      return new Response(
        JSON.stringify({
          success: false,
          error: "No RSVPs found",
        }),
        {
          status: 404,
          headers,
        }
      );
    }

    const rsvpList = csvToObjects(fileContent);
    
    // Find exact match by name and timestamp
    const updatedList = rsvpList.filter(
      (rsvp) => !(rsvp.name === name && rsvp.timestamp === timestamp)
    );

    if (updatedList.length === rsvpList.length) {
      return new Response(
        JSON.stringify({
          success: false,
          error: "RSVP not found",
        }),
        {
          status: 404,
          headers,
        }
      );
    }

    const csvContent = objectsToCSV(updatedList);
    await putS3Data(FILE_KEY, csvContent, "text/csv");

    return new Response(JSON.stringify({ success: true }), {
      status: 200,
      headers,
    });
  } catch (error) {
    console.error("Error deleting RSVP:", error);
    return new Response(
      JSON.stringify({
        success: false,
        error: error instanceof Error ? error.message : "Unknown error",
      }),
      {
        status: 500,
        headers,
      }
    );
  }
};

// Export protected routes
export const GET = createProtectedAPIRoute(handleGet, "admin");
export const POST = createProtectedAPIRoute(handlePost, "guest");
export const DELETE = createProtectedAPIRoute(handleDelete, "admin");
