import { jwtDecode } from "jwt-decode";

interface JWTPayload {
  role: string;
  iat?: number;
  exp?: number;
}

export function setAuthData(token: string) {
  sessionStorage.setItem('authToken', token);
  
  // Dispatch auth event
  const payload = jwtDecode<JWTPayload>(token);
  const event = new CustomEvent("auth-success", {
    bubbles: true,
    composed: true,
    detail: { role: payload?.role }
  });
  document.dispatchEvent(event);
}

export function clearAuthData() {
  sessionStorage.removeItem('authToken');
  window.location.reload();
}

export function getAuthToken(): string | null {
  return sessionStorage.getItem('authToken');
}

export function getRole(): string | null {
  const token = getAuthToken();
  if (!token) return null;
  
  try {
    const payload = jwtDecode<JWTPayload>(token);
    // Check if token is expired
    if (payload.exp && payload.exp * 1000 < Date.now()) {
      clearAuthData(); // Clear expired token
      return null;
    }
    return payload.role;
  } catch (error) {
    console.error('Error decoding token:', error);
    clearAuthData(); // Clear invalid token
    return null;
  }
}

export function isAuthenticated(): boolean {
  return getRole() !== null;
}

export function hasRole(requiredRole: "guest" | "admin"): boolean {
  const role = getRole();
  if (!role) return false;
  if (requiredRole === "admin") return role === "admin";
  if (requiredRole === "guest") return role === "guest" || role === "admin";
  return false;
} // admin can access guest pages, but guest can't access admin pages

export async function fetchWithAuth(url: string, options: RequestInit = {}): Promise<Response> {
  const token = getAuthToken();
  if (!token) {
    throw new Error('No authentication token found');
  }

  const headers = {
    ...options.headers,
    'Authorization': `Bearer ${token}`,
  };

  return fetch(url, {
    ...options,
    headers,
  });
} 