import type { APIContext, APIRoute } from "astro";
import { verifyToken, extractTokenFromHeader } from "./jwt";

export async function validateAuth(
  context: APIContext,
  requiredRole?: "guest" | "admin"
): Promise<{ isValid: boolean; role?: string; error?: string }> {
  const authHeader = context.request.headers.get("Authorization");
  const token = extractTokenFromHeader(authHeader || "");

  if (!token) {
    return { isValid: false, error: "No token provided" };
  }

  const payload = verifyToken(token);
  if (!payload) {
    return { isValid: false, error: "Invalid token" };
  }

  if (requiredRole && payload.role !== requiredRole) {
    if (requiredRole === "admin" && payload.role !== "admin") {
      return { isValid: false, error: "Admin access required" };
    }
  }

  return { isValid: true, role: payload.role };
}

export function createProtectedAPIRoute(handler: Function, requiredRole?: "guest" | "admin"): APIRoute {
  return async (context: APIContext) => {
    const { isValid, error } = await validateAuth(context, requiredRole);

    if (!isValid) {
      return new Response(
        JSON.stringify({
          success: false,
          error: error || "Authentication failed",
        }),
        {
          status: 401,
          headers: {
            "Content-Type": "application/json",
          },
        }
      );
    }

    return handler(context);
  };
} 